#! /usr/bin/tclsh
#
#######################################################################################################
#
#	Tcl script v1.0 implementing CGI interface for sending APRS messages via html page.
#	Written by Max Monti ik3svw@ir3ip.net http://www.ir3ip.net/ik3svw -  sep 2004
#
#	Please put this file in the same WEB public folder where the aprsmsg.html page is stored.
#
#	Warning about aprsmsg system: if you let others to send messages wia web aprs message html page,
#	then they can send packets to internet using your main Call (SysCall) set in aprsmsg.tcl script.
#	So you are still responsible for any packet sent with your main Call thru aprsmsg.tcl script even
#	if someone else is sending packets unsing this web aprs message cgi interface.
#
#######################################################################################################
#
#
#		Please change main parameters below to let this interface to tork correctly.
#
#################### START -  MAIN PARAMETERS TO BE CHANGED WITH APPROPIATE VALUES ####################
#
set SysCall "NOCALL"; 				# Main Call - Must be same as SysCall in aprsmsg.TCL script
#
set APRSMSG_FILE "../Aprs_msg";	 	# Path and file name for storing APRS Message - Must be same as APRSMSG_FILE in aprsmsg.TCL script
set COUNTER_FILE "./Aprsmsg_counter";	# Path and file name for internal message counter
#
set AprsdCall "SVRCALL"; 			# Aprs server call to be shown in html returned page
set HamServer "<a href=\"http://www.WebServer.net\">Web Server"; # Home page htm link of Web Server (note \" instead of ")
#
##################### END -  MAIN PARAMETERS TO BE CHANGED WITH APPROPIATE VALUES #####################
#
#
#
#
###### The following code has no user settings - do not change it ######
#
#
set par(nodata) 0
#
#
############## Parse CGI parameters stored in message ################
###### This procedure has no user settings - do not change code ######
proc parsepar {delim message} {
    global par
    foreach query [split $message $delim] {
	set index [string first "=" $query]
	set parname [string range $query 0 [expr $index-1]]
	set parvalue [string map {"%25" "+++"} [string map {"+" " "} [string range $query [expr $index+1] end]]]
	while 1 {
	    set index [string first "%" $parvalue]
	    if "$index == -1" {
		break
	    } else {
		set hex [string range $parvalue [expr $index+1] [expr $index+2]]
		regsub -all -nocase "\%$hex" $parvalue [format "%c" "0x$hex"] parvalue
	    }
	}
	set parvalue [string map {"+++" "%"} $parvalue]
	set par($parname) $parvalue
    }
}
#
#
####### An error occurred before sending HTML page: write this error in HTML format #########
################## This procedure has no user settings - do not change code #################
# write error output to stdout in a html page
proc printerror {string} {
	global SysCall
	puts stdout "<center>This ERROR was generated by $SysCall \
	service:<p>$string</p></CENTER>"
}
#
#
################################ main input ############################
###### The following code has no user settings - do not change it ######
# read input cgi parameters
if {![info exist env(REQUEST_METHOD)]} {
    puts stdout "Content-type: text/html;\n\n"
    puts stdout "<html><head><title>Error APRS message</title></head><body>"
    printerror "Invalid cgi parameters (REQUEST_METHOD not set)"
    puts stdout "<center><B><A HREF=\"JavaScript:window.history.go(-1)\">Back</A></B></center>"
    puts stdout "</body></html>"
    return
}
# read input in case of PUT data
set message $env(QUERY_STRING)
if {[string length $message]>0} {
    parsepar "&" $message
}
# read input in case of POST data
if {[string equal $env(REQUEST_METHOD) "POST"]} {
    set message [read stdin $env(CONTENT_LENGTH)]
    parsepar "\n" $message
}
#
#
#
#
############################# main output ##############################
###### The following code has no user settings - do not change it ######
# print html header
puts stdout "Content-type: text/html;\n\n"
puts stdout "<html><head><title>APRS message submission</title></head><body bgcolor=\"#ACCCCC\">"
puts stdout "<center><hr><H2>$SysCall APRS message service</h2></center><hr>"
# does counter filename exist ?
if {[file readable $COUNTER_FILE] && [file writable $COUNTER_FILE] && [file size $COUNTER_FILE]==8} {
	# open counter file
	set fd [open $COUNTER_FILE RDWR]
	scan [read $fd 8] "%x" counter
	seek $fd 0 start
	puts -nonewline $fd [format "%08x" [expr $counter+1]]
	close $fd
} else {
	# Counter don't exist or is not correct: create it !!
	set counter 0
	if {[catch "open $COUNTER_FILE {RDWR CREAT} 0644" fd] != 0} {
		printerror "Error creating counter"
	} else {
	puts -nonewline $fd [format "%08x" [expr $counter+1]]
	close $fd
	}
}
# are all input data non-blank ?
if {[info exist par(MyCall)] && [info exist par(ToCall)] && [info exist par(MSG)]} {
	# length check on input data
	if {[string bytelength $par(MyCall)]<4 || [string bytelength $par(ToCall)]<4 || [string bytelength $par(MSG)]<4} {
		# form not compiled correctly
		printerror "Not all parameters have been received correctly"
	} else {
		# default
		set NP [split $par(ToCall) "|"]
		if {[catch "open $APRSMSG_FILE {WRONLY CREAT APPEND} 0666" fdm]} {
			printerror "File Error: $fdm"
		} else {
			set Ts [string repeat " " [expr 9 - [string length [lindex $NP 0]]]]
			puts $fdm "[lindex $NP 0]|$par(MyCall)>$par(MSG)"
			close $fdm
			set Message "$SysCall>APRS,TCPIP,$AprsdCall:<br>:[lindex $NP 0]$Ts:$par(MyCall)>$par(MSG)"
			puts stdout "<font face=\"arial\"><p align=\"center\"><b><i>The following APRS message:</i></p><p align=\"center\"><table border=\"1\" width=\"90%\"><tr><td><small>$Message</small></td></tr></table></p><p align=\"center\"><i>has been queued to the local VHF/TCP APRS port.</i></b></p></font>"
		}
	}
} else {
	# parameters missing
	printerror "Missing one ore more parameters in input data<br>$message"
}
# print html footer
puts stdout "<hr><center><p><table border=\"1\" width=\"100%\"><tr><td width=\"33%\" align=\"center\"><A HREF=\"JavaScript:window.history.go(-1)\">Back</A></td><td width=\"33%\" align=\"center\"><a href=\"aprsmsghelp.html\">$SysCall Info</a></td><td width=\"33%\" align=\"center\">Powered by $HamServer ham server</a></td></tr></table></p>"
puts stdout "<p>&nbsp;</p><p><small>$SysCall processed messages: $counter</small></p></center>"
puts stdout "</body></html>"